#!/bin/sh
#
# Example backup script:
# - partitions of main server:
#   sda1, sdb1, sdc1, sdd1: swap partitions (no backup)
#   sda2: root partition, here is the operating system located
#   sdb2, sdc2, sdd2: data partitions
# - partitions of backup server:
#   hda1: swap partition
#   hda2: root partition, operating system only
#   hda3: backup data, inclusive the operating system of the main server
#         This partition is exported using NFS; therefore the main server
#         can import this partition using the NFS device
#         192.168.0.2:/mnt/hda3, where 192.168.0.2 is the IP address of
#         the backup machine
#
# The backup of the root partition includes all OS directories except
# /proc and /mnt, because here other file systems are mounted; the
# directory structure of /proc and /mnt is created using mkdir on the
# backup machine.
#

echo "Beginning Backup at "`date`

# mounting backup partition if possible
if mount 192.168.0.2:/mnt/hda3 /mnt/backup/
then

# deleting old backup data
rm -rf /mnt/backup/{sda2,sdb2,sdc2,sdd2}

# creating destination directories for the main server partitions
mkdir /mnt/backup/{sda2,sdb2,sdc2,sdd2}

# creating /proc and /mnt directory structure
mkdir /mnt/backup/sda2/mnt /mnt/backup/sda2/proc
mkdir /mnt/backup/sda2/mnt/{backup,cdrom,floppy,sdb2,sdc2,sdd2}

# copying operating system partition
cp -Rdp /bin /boot /dev /etc /home /lib /opt /root /sbin /tmp /usr /var /mnt/backup/sda2/

# copying data partitions
cp -Rdp /mnt/sdb2/* /mnt/backup/sdb2/
cp -Rdp /mnt/sdc2/* /mnt/backup/sdc2/
cp -Rdp /mnt/sdd2/* /mnt/backup/sdd2/

# unmounting backup partition
umount /mnt/backup/

echo "Finishing Backup at "`date`

else
echo "Mounting not possible, stopping at "`date`
fi
