#!/bin/sh
########################################################################
# File:           publicfiles/updatedd_wrapper.sh                      #
# myLinux Server: Copyright (c) 2003 Michael Oberg                     #
# Version:        0.96                                                 #
# Author:         Michael Oberg <michael.oberg@mylinuxproject.de>      #
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License as published by #
# the Free Software Foundation; either version 2 of the License, or    #
# (at your option) any later version.                                  #
#                                                                      #
# This program is distributed in the hope that it will be useful,      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of       #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         #
# GNU General Public License for more details.                         #
#                                                                      #
# You should have received a copy of the GNU Public License along      #
# with this package; if not, write to the Free Software Foundation,    #
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.       #
########################################################################

#
# This is a wrapper for updatedd's rc scripts. It checks the servers
# externel IP address for changes and calls the updatedd rc script only
# if the IP address has changed since the last call of the wrapper or
# (f.e. for the first run) is not the same as the one your dynamic DNS
# delivers.
#

# Name of your server at your dynamic DNS provider
EXTERNALSERVERNAME="mylinuxproject.dyndns.org"

# interface which gets the dynamic address (generally ppp0)
INTERFACE=ppp0

# path/name of your updatedd script
UPDATEDDSCRIPT="/usr/local/sbin/mylinux/rc_updatedd_dyndns"



if [ -e /tmp/updatedd_wrapper_address ]
then
  OLDADDRESS=`cat /tmp/updatedd_wrapper_address`
else
  # Script seems to be called for the first time or after a
  # "clean sweep" action of the OS.
  # Detect the address of your dynamic DNS provider first.
  OLDADDRESS=`dig "$EXTERNALSERVERNAME" | grep "^$EXTERNALSERVERNAME" | sed 's/.*[ \t]\([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*\)$/\1/'`
  # store the current address for the next run
  echo -n "$OLDADDRESS" > /tmp/updatedd_wrapper_address
fi

# detection of the current address
CURRENTADDRESS=`ifconfig $INTERFACE | grep 'inet addr:' | sed 's/.*inet addr:\([0-9.]*\).*/\1/'`

if [ ! "$OLDADDRESS" = "$CURRENTADDRESS" ]
then
  # call updatedd
  $UPDATEDDSCRIPT
  # store the current address for the next run
  echo -n "$CURRENTADDRESS" > /tmp/updatedd_wrapper_address
fi
