#!/bin/sh
########################################################################
# File:           setup/samba.sh                                       #
# myLinux Server: Copyright (c) 2003 Michael Oberg                     #
# Version:        0.97                                                 #
# Author:         Michael Oberg <michael.oberg@mylinuxproject.de>      #
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License as published by #
# the Free Software Foundation; either version 2 of the License, or    #
# (at your option) any later version.                                  #
#                                                                      #
# This program is distributed in the hope that it will be useful,      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of       #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         #
# GNU General Public License for more details.                         #
#                                                                      #
# You should have received a copy of the GNU Public License along      #
# with this package; if not, write to the Free Software Foundation,    #
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.       #
########################################################################
source mylinux.conf
TOPDOMAIN=`echo $DOMAIN | sed 's/.*\.\([^.]*\)$/\1/'`
SUBDOMAIN=`echo $DOMAIN | sed 's/\(.*\)\.[^.]*$/\1/'`
LDAPBASE="dc=$SUBDOMAIN,dc=$TOPDOMAIN"
##############################################################
# samba configuration                                        #
##############################################################

cat << EOF > /etc/samba/smb.conf
# Global parameters
[global]
	unix charset = ISO-8859-15
	display charset = ISO-8859-15
	workgroup = $WINDOMAIN
	map to guest = Bad User
EOF

if [ x$WINDOWSSERVER != x ]
then
  cat << EOF >> /etc/samba/smb.conf
	security = SERVER
	password server = $WINDOWSSERVER
EOF
fi

cat << EOF >> /etc/samba/smb.conf
	log file = /var/log/samba
	keepalive = 30
EOF

if [ x$WINDOWSSERVER = x ]
then
  cat << EOF >> /etc/samba/smb.conf
	add machine script = $BINPATH/machineadd.pl %u
	logon script = login.bat
	logon drive = $HOMEDRIVE
	domain logons = Yes
	os level = 64
	preferred master = Yes
	domain master = Yes
	wins support = Yes
	ldap server = $LDAPSERVER
	ldap port = 389
	ldap suffix = $LDAPBASE
	ldap admin dn = cn=admin,$LDAPBASE
	ldap ssl = start tls
	ldap passwd sync = Yes
	passdb backend = ldapsam:"ldap://$LDAPSERVER"
EOF
fi

cat << EOF >> /etc/samba/smb.conf
	printing = cups
	printcap name = cups
	kernel oplocks = No
	create mask = 0660
	directory mask = 0770
	socket options = SO_KEEPALIVE SO_BROADCAST TCP_NODELAY

EOF

if [ x$WINDOWSSERVER = x ]
then
  cat << EOF >> /etc/samba/smb.conf
[netlogon]
	comment = Windows NT NetLogon Service
	path = /var/lib/samba/netlogon
	read only = Yes

EOF
fi

if [ ! -d "$TEMPLATEPATH" ]
then
  mkdir -m 2775 -p "$TEMPLATEPATH"
fi
chgrp -R users "$TEMPLATEPATH"
if [ ! -d "$DOCUMENTSPATH" ]
then
  mkdir -m 2770 -p "$DOCUMENTSPATH"
fi
chgrp -R users "$DOCUMENTSPATH"
if [ ! -d "$SOFTWAREPATH" ]
then
  mkdir -m 2755 -p "$SOFTWAREPATH" \
		   "$SOFTWAREPATH/Admin" \
		   "$SOFTWAREPATH/Admin/PuTTY" \
		   "$SOFTWAREPATH/Images" \
		   "$SOFTWAREPATH/Tools" \
		   "$SOFTWAREPATH/Multimedia" \
		   "$SOFTWAREPATH/Windows" \
		   "$SOFTWAREPATH/Windows/Internet Explorer" \
		   "$SOFTWAREPATH/Windows/Default User" \
		   "$SOFTWAREPATH/Windows/Fonts" \
		   "$SOFTWAREPATH/Office Service Packs" \
		   "$SOFTWAREPATH/Drivers" \
		   "$SOFTWAREPATH/MSOffice" \
		   "$SOFTWAREPATH/MSPublisher"
fi
chgrp -R users "$SOFTWAREPATH"
cat << EOF > "$SOFTWAREPATH/Windows/AddDomainGroups.cmd"
rem German version:
net localgroup Administratoren "$WINDOMAIN\\Domain Admins" /add
net localgroup Benutzer "$WINDOMAIN\\Domain Users" /add
net localgroup Gste "$WINDOMAIN\\Domain Guests" /add
rem US version:
net localgroup Administrators "$WINDOMAIN\\Domain Admins" /add
net localgroup Users "$WINDOMAIN\\Domain Users" /add
net localgroup Guests "$WINDOMAIN\\Domain Guests" /add
pause
EOF
cat << EOF > "$SOFTWAREPATH/Windows/nettime.cmd"
net time \\\\$HOSTNAME /set /yes
EOF

cat << EOF >> /etc/samba/smb.conf
[$TEMPLATESHARE]
	comment = Microsoft Office Templates
	path = $TEMPLATEPATH
	#vfs object = vscan-clamav
	#vfs options = config-file = /etc/samba/vscan-clamav.conf
	read only = No
	create mask = 0664
	directory mask = 0775

[$SOFTWARESHARE]
	comment = Windows Software
	path = $SOFTWAREPATH
	#vfs object = vscan-clamav
	#vfs options = config-file = /etc/samba/vscan-clamav.conf
	read only = No
	create mask = 0644
	directory mask = 0755

[$DOCUMENTSSHARE]
	comment = Documents
	path = $DOCUMENTSPATH
	#vfs object = vscan-clamav
	#vfs options = config-file = /etc/samba/vscan-clamav.conf
	read only = No

EOF

if [ x$WINDOWSSERVER = x ]
then
  cat << EOF >> /etc/samba/smb.conf
[homes]
	comment = Users Home Directory
	#vfs object = vscan-clamav
	#vfs options = config-file = /etc/samba/vscan-clamav.conf
	read only = No
	browseable = No

EOF
fi

cat << EOF >> /etc/samba/smb.conf
[printers]
	comment = All Printers
	path = /tmp
	create mask = 0700
	printable = Yes
	browseable = No

[print$]
	comment = Upload Directory for Printer Drivers
# Warning: Virus scanning cannot be activated (at least this
# was the case with Samba 2.2.8a). At installing new drivers
# these drivers will be placed in a wrong location.
#	vfs object = vscan-clamav
#	vfs options = config-file = /etc/samba/vscan-clamav.conf
	path = /var/lib/samba/printers
	write list = root
	guest ok = Yes
	create mask = 0644
	directory mask = 0755

[pdfdocs]
	comment = Output directory for the PDF printer
	#vfs object = vscan-clamav
	#vfs options = config-file = /etc/samba/vscan-clamav.conf
	path = /var/lib/samba/pdfdocs
	create mask = 666
	directory mask = 0777
	read only = No

[PdfPrinter]
	comment = PdfPrinter
	path = /tmp
	printer admin = root
	read only = No
	create mask = 0700
	printable = Yes
	printer name = PdfPrinter
	oplocks = No
	share modes = No
EOF

cat /usr/local/sbin/mylinux/login.bat \
  | sed -e "s/#WEBSERVER#/$WEBSERVER/g" \
        -e "s/#HOMEDRIVE#/$HOMEDRIVE/g" \
        -e "s/#TEMPLATEDRIVE#/$TEMPLATEDRIVE/g" \
        -e "s/#TEMPLATESHARE#/$TEMPLATESHARE/g" \
        -e "s/#DOCUMENTSDRIVE#/$DOCUMENTSDRIVE/g" \
        -e "s/#DOCUMENTSSHARE#/$DOCUMENTSSHARE/g" \
        -e "s/#SOFTWAREDRIVE#/$SOFTWAREDRIVE/g" \
        -e "s/#SOFTWARESHARE#/$SOFTWARESHARE/g" \
  > /var/lib/samba/netlogon/login.bat
