#!/bin/sh
########################################################################
# File:           setup/sendmail.sh                                    #
# myLinux Server: Copyright (c) 2003 Michael Oberg                     #
# Version:        0.98                                                 #
# Author:         Michael Oberg <michael.oberg@mylinuxproject.de>      #
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License as published by #
# the Free Software Foundation; either version 2 of the License, or    #
# (at your option) any later version.                                  #
#                                                                      #
# This program is distributed in the hope that it will be useful,      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of       #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         #
# GNU General Public License for more details.                         #
#                                                                      #
# You should have received a copy of the GNU Public License along      #
# with this package; if not, write to the Free Software Foundation,    #
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.       #
########################################################################
source mylinux.conf
TOPDOMAIN=`echo $DOMAIN | sed 's/.*\.\([^.]*\)$/\1/'`
SUBDOMAIN=`echo $DOMAIN | sed 's/\(.*\)\.[^.]*$/\1/'`
##############################################################
# create sendmail.conf                                       #
##############################################################
# Requires: the sendmail daemon has to be stopped before
# running this script
echo "$HOSTNAME" > /etc/mail/local-host-names

pushd /usr/src/sendmail/cf/cf/ > /dev/null

# create m4 configuration script
cat << EOF > sendmail.mc
divert(-1)
#
# Copyright (c) 1998, 1999 Sendmail, Inc. and its suppliers.
#       All rights reserved.
# Copyright (c) 1983 Eric P. Allman.  All rights reserved.
# Copyright (c) 1988, 1993
#       The Regents of the University of California.  All rights reserved.
# Changed 2003 by Michael Oberg
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
#

#
#  This is a generic configuration file for Linux.
#  It has support for local and SMTP mail only.  If you want to
#  customize it, copy it to a name appropriate for your environment
#  and do the modifications there.
#

divert(0)dnl
VERSIONID(\`: generic-linux.mc,v 8.1 1999/09/24 22:48:05 gshapiro Exp $')
OSTYPE(linux)dnl
FEATURE(\`always_add_domain')
FEATURE(\`ldap_routing')
define(\`confLDAP_DEFAULT_SPEC',\`-h $LDAPSERVER -b dc=$SUBDOMAIN,dc=$TOPDOMAIN')
define(\`confLDAP_CLUSTER',\`Mailserver')
define(\`ALIAS_FILE',\`ldap:')
FEATURE(\`access_db',\`LDAP')
FEATURE(\`virtusertable',\`LDAP')
LDAPROUTE_DOMAIN(\`$DOMAIN')
MAILER(local)
EOF

if [ $IMAPSERVER = $FQHOSTNAME ]
then
  # only on the IMAP server itself mail can be delivered using
  # the local LMTP socket - which is the method defined in cyrusv2
  cat << EOF >> sendmail.mc
MAILER(cyrusv2)
define(\`confLOCAL_MAILER',\`cyrusv2')
LOCAL_RULE_0
R$=N			$: 0local $: 
R$=N < @ $=w . >	$: 0local $: 
Rbb + $+ < @ $=w . >	0cyrusbb $: 
EOF
  # in cyrusv2 the path to the lmtp socket is defined
  if [ ! -e ../mailer/cyrusv2.m4-template ]
  then
    mv ../mailer/cyrusv2.m4 ../mailer/cyrusv2.m4-template
  fi
  cat ../mailer/cyrusv2.m4-template | sed "s@/var/imap@$MAILDBPATH@" > ../mailer/cyrusv2.m4
fi

cat << EOF >> sendmail.mc
MAILER(\`smtp')dnl
MAILER(\`procmail')dnl
MAILER(\`uucp')dnl
DOMAIN(\`generic')dnl
LOCAL_CONFIG
Cwlocalhost $DOMAIN $ADDITIONALMAILDOMAINS
FEATURE(accept_unresolvable_domains)
define(\`CERT_DIR',\`/etc/mail/certs')
define(\`confCACERT_PATH',\`/etc/ssl/mylinux')
define(\`confCACERT',\`confCACERT_PATH/cacert.pem')
define(\`confSERVER_CERT',\`CERT_DIR/host.cert')
define(\`confSERVER_KEY',\`CERT_DIR/host.key')
define(\`confCLIENT_CERT',\`CERT_DIR/host.cert')
define(\`confCLIENT_KEY',\`CERT_DIR/host.key')
define(\`confAUTH_MECHANISMS',\`LOGIN PLAIN')
TRUST_AUTH_MECH(\`LOGIN PLAIN')
HACK(\`dnswl', \`whitelist.$DOMAIN')dnl
FEATURE(\`dnsbl',\`dnsbl.sorbs.net',\`"554 Rejected " \$&{client_addr} " found in dnsbl.sorbs.net"')dnl
FEATURE(\`delay_checks')
EOF

if [ $IMAPSERVER = $FQHOSTNAME ]
then
  # only if mail is delivered locally, scan - so scanning is
  # enabled only on the IMAP server, double scanning is avoided
  cat << EOF >> sendmail.mc
INPUT_MAIL_FILTER(\`clamav', \`S=local:/usr/share/clamav-sockets/clamav-milter.sock, F=, T=S:4m;R:4m')
define(\`confINPUT_MAIL_FILTERS', \`clamav')
EOF
else
  # if this is not the IMAP server, the mail has to be forwarded
  # to this server
  cat << EOF >> sendmail.mc
define(\`MAIL_HUB', \`relay:$IMAPSERVER')
EOF
fi

if [ x$SMARTHOST != x ]
then
  if [ x$SMARTHOST_USER != x ]
  then
    cat << EOF >> sendmail.mc
FEATURE(\`authinfo', \`hash /etc/mail/authinfo')
EOF
    cat << EOF > /etc/mail/authinfo
AuthInfo:$SMARTHOST "U:$SMARTHOST_USER" "P:$SMARTHOST_PW"
EOF
  chmod 600 /etc/mail/authinfo
  makemap hash /etc/mail/authinfo.db < /etc/mail/authinfo
  fi
    cat << EOF >> sendmail.mc
define(\`SMART_HOST', \`$SMARTHOST')
EOF
fi

sh Build sendmail.cf
sh Build install-cf
popd > /dev/null

echo "Remark: you may whitelist foreign mailservers in your nameserver config,"
echo "in the file /home/named/etc/namedb/pz/whitelist.$DOMAIN.hosts"
