package usermanager;
########################################################################
# File:           usermanager.pm                                       #
# myLinux Server: Copyright (c) 2003 Michael Oberg                     #
# Version:        0.98                                                 #
# Author:         Michael Oberg <michael.oberg@mylinuxproject.de>      #
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License as published by #
# the Free Software Foundation; either version 2 of the License, or    #
# (at your option) any later version.                                  #
#                                                                      #
# This program is distributed in the hope that it will be useful,      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of       #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         #
# GNU General Public License for more details.                         #
#                                                                      #
# You should have received a copy of the GNU Public License along      #
# with this package; if not, write to the Free Software Foundation,    #
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.       #
########################################################################

use mylinux::env;
use Net::Telnet;
use User::pwent;
use FileHandle;
use IPC::Open2;


# procedures for cyrus administration (email)

sub createmailbox {
  my ($LOGINNAME, $PASSWD) = @_;

  $hostname = $env::imapserver;
  $username = "cyrus";
  if (!(-r "$env::DATAPATH/cyruspasswd")) {
    die "$env::DATAPATH/cyruspasswd is not readable!\n";
  }
  $passwd = `cat $env::DATAPATH/cyruspasswd`;

  # Open up a new session to imap server
  $imap = new Net::Telnet (Telnetmode => 0);
  $imap->open (Host => $hostname, Port => 143);

  # Read connection message.
  $line = $imap->getline;
  die "creation of mailbox failed!\n$line\n" unless $line =~ /^\* OK/;
  print "connected to imap server ok\n";

  # Send admin user name.
  $imap->print (". login $username $passwd");
  $line = $imap->getline;
  die "creation of mailbox failed!\n$line\n" unless $line =~ /^\. OK/;
  print "logged in to imap server as user cyrus\n";

  # Send create command.
  $imap->print (". create user.$LOGINNAME");
  $line = $imap->getline;
  die "creation of mailbox failed!\n$line\n" unless $line =~ /^\. OK/;
  print "created new user $LOGINNAME\n";
  $imap->print (". setacl user.$LOGINNAME $username lrswipcda");
  $imap->print (". setacl user.$LOGINNAME cyrus lrswipcda");
  $line = $imap->getline;
  die "creation of mailbox failed!\n$line\n" unless $line =~ /^\. OK/;
  print "permissions for user and administrator set\n";
  $imap->print (". create \"user.$LOGINNAME.$env::mailtrash\"");
  $imap->print (". create \"user.$LOGINNAME.$env::mailsent\"");
  $imap->print (". create \"user.$LOGINNAME.$env::maildrafts\"");
  $line = $imap->getline;
  print "warning: creation of mailbox subfolders failed\n$line\n" unless $line =~ /^\. OK/;
  print "subfolders created\n";

  $imap->print (". logout");

  # Open up a new session to imap server
  $imap->open (Host => $hostname, Port => 143);

  # Read connection message.
  $line = $imap->getline;
  print "warning: subscription to mailbox failed\n$line\n" unless $line =~ /^\* OK/;
  print "reconnected to imap server ok\n";

  # Send user name.
  $imap->print (". login $LOGINNAME $PASSWD");
  $line = $imap->getline;
  print "warning: subscription to mailbox failed\n$line\n" unless $line =~ /^\. OK/;
  print "logged in to imap server as user $LOGINNAME\n";

  $imap->print (". subscribe \"user.$LOGINNAME.$env::mailtrash\"");
  $imap->print (". subscribe \"user.$LOGINNAME.$env::mailsent\"");
  $imap->print (". subscribe \"user.$LOGINNAME.$env::maildrafts\"");
  $line = $imap->getline;
  print "warning: subscription to mailbox failed\n$line\n" unless $line =~ /^\. OK/;
  print "subfolders subscribed\n";

  $imap->print (". logout");
}

sub deletemailbox {
  my ($LOGINNAME) = @_;

  $hostname = $env::imapserver;
  $username = "cyrus";
  if (!(-r "$env::DATAPATH/cyruspasswd")) {
    die "$env::DATAPATH/cyruspasswd is not readable!\n";
  }
  $passwd = `cat $env::DATAPATH/cyruspasswd`;

  # Open up a new session to imap server
  $imap = new Net::Telnet (Telnetmode => 0);
  $imap->open (Host => $hostname, Port => 143);

  # Read connection message.
  $line = $imap->getline;
  die "deletion of mailbox failed!\n$line\n" unless $line =~ /^\* OK/;
  print "connected to imap server ok\n";

  # Send admin user name.
  $imap->print (". login $username $passwd");
  $line = $imap->getline;
  die "deletion of mailbox failed!\n$line\n" unless $line =~ /^\. OK/;
  print "logged in to imap server as user cyrus\n";

  $imap->print(qq(. list "" user.$LOGINNAME*));
  $i = 0;
  $line = $imap->getline;
  while (!($line =~ /^\. OK/)) {
    if ($line =~ /user\.$LOGINNAME(.*)"/) {
      $good[$i++] = "user.$LOGINNAME$1";
    }
    $line = $imap->getline;
  }
  foreach (reverse @good) {
    $imap->print(". delete \"$_\"");
    $line = $imap->getline;
    die $line unless $line =~ /^\. OK/;
    print "deleted $_ folder\n";
  }

  print "user deleted.\n";
  $imap->print (". logout");
}

# procedures for unix administration

sub getgidnumber {
  my $max = 1000;
  foreach my $x (split /\n/, `ldapsearch -b "$env::ldapbase" "*" -h $env::ldapserver -ZZ | grep gidNumber | sed 's/gidNumber: //'`) {
    if ($max < $x+0) { $max = $x+0; }
  }
  return $max+1;
}

sub getuidnumber {
  my $max = 1000;
  foreach my $x (split /\n/, `ldapsearch -b "$env::ldapbase" "*" -h $env::ldapserver -ZZ | grep uidNumber | sed 's/uidNumber: //'`) {
    if ($max < $x+0) { $max = $x+0; }
  }
  return $max+1;
}

sub createmachine {

  my ($LOGINNAME, $TMPUID, $TMPGID) = @_;

  if (getpwnam ($LOGINNAME)) {
    die ("Machine $LOGINNAME already exists!\n");
  }

  if (-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME") {
    $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME`;
    if (-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME") {
      die ("Cannot delete existing file $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME!\n");
    }
  }

  $cmd = qq|/bin/cat > $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME << EOF\n|
       . qq|dn: uid=$LOGINNAME, ou=machines, $env::ldapbase\n|
       . qq|cn: $LOGINNAME\n|
       . qq|sn: $LOGINNAME\n|
       . qq|uid: $LOGINNAME\n|
       . qq|uidNumber: $TMPUID\n|
       . qq|gidNumber: $TMPGID\n|
       . qq|homeDirectory: /dev/null\n|
       . qq|loginShell: /bin/false\n|
       . qq|userPassword: {SSHA}!\n|
       . qq|description: MS Windows machine account\n|
       . qq|objectClass: person\n|
       . qq|objectClass: posixAccount\n|
       . qq|EOF\n|
       . qq|chmod 600 $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME\n|;

  $TMPOUT = `$cmd`;

  if (!(-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME")) {
    die ("Cannot create file $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME!\n");
  }

  $TMPOUT = `/usr/bin/ldapadd -ZZ -a -c -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver` or die ("ldapadd failed!\n");

  $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME`;

}

sub deletemachine {

  my ($LOGINNAME) = @_;

  if (!getpwnam ($LOGINNAME)) {
    die ("Machine $LOGINNAME doesn't exist.\n");
  }

  $TMPOUT = `/usr/bin/ldapsearch -x -b "$env::ldapbase" -LLL "(uid=$LOGINNAME)"`;
  $TMPOUT = "\n" . $TMPOUT;

  $TMPOUT =~ /\ndn: ([^\n]*)\n/si;
  $DN = $1;

  $TMPOUT = `/usr/bin/ldapdelete -ZZ -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver "$DN"`;

  print "machine deleted\n";

}

sub deletegroup {

  my ($GROUPNAME) = @_;

  if (!getgrnam($GROUPNAME)) {
    die "Group $GROUPNAME doesn't exist.\n";
  }

  $TMPOUT = `/usr/bin/ldapsearch -x -b "ou=groups, $env::ldapbase" -LLL "(cn=$GROUPNAME)"`;
  $TMPOUT = "\n" . $TMPOUT;

  $TMPOUT =~ /\ndn: ([^\n]*)\n/si;
  $DN = $1;

  $TMPOUT = `/usr/bin/ldapdelete -ZZ -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver "$DN"`;

  print "group deleted\n";

}

sub creategroup {

  my ($GROUPNAME, $TMPGID, $DESCRIPTION, @MEMBERLIST) = @_;

  my ($GROUPNAME) = @_;

  if (getgrnam($GROUPNAME)) {
    die "Group $GROUPNAME already exists!\n";
  }

  if (-e "$env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME") {
    $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME`;
    if (-e "$env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME") {
      die ("Cannot delete existing file $env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME!\n");
    }
  }

  $DESCRIPTION = pack ("U*", unpack ("C*", $DESCRIPTION));
  $DESCRIPTION64 = `echo -n "$DESCRIPTION" | base64 -e`;
  $DESCRIPTION64 =~ /^([a-zA-Z0-9+\/=]*)/; $DESCRIPTION64 = $1;

  $cmd  = qq|/bin/cat > $env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME << EOF\n|
        . qq|dn: cn=$GROUPNAME, ou=groups, $env::ldapbase\n|
        . qq|cn: $GROUPNAME\n|
        . qq|gidNumber: $TMPGID\n|
        . qq|description:: $DESCRIPTION64\n|
        . qq|objectClass: posixGroup\n|;

  foreach $MEMBER (@MEMBERLIST) {
    $cmd .= qq|memberUid: $MEMBER\n|
  }

  $cmd .= qq|EOF\n|
        . qq|chmod 600 $env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME\n|;

  $TMPOUT = `$cmd`;

  if (!(-e "$env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME")) {
    die ("Cannot create file $env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME!\n");
  }

  $TMPOUT = `/usr/bin/ldapadd -ZZ -a -c -f $env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver` or die ("ldapadd failed!\n");

  $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEGROUP-$GROUPNAME`;

  $TMPOUT = `/usr/bin/net groupmap add ntgroup=$GROUPNAME UNIXgroup=$GROUPNAME`;

  print "group created\n";

}

sub create {

  my ($FIRSTNAME, $MIDDLENAME, $LASTNAME, $LOGINNAME, $TMPUID, $TMPGID, $HOMEDIR, $SHORTEMAIL, $TMPPW) = @_;

  if (getpwnam ($LOGINNAME)) {
    die ("User $LOGINNAME already exists!\n");
  }

  $TMPPW =~ s/\r//g;
  $TMPPW =~ s/\n//g;
  $SMBPW = $TMPPW;		# Backup needed by samba
  $TMPPW =~ s/'/'\\''/g;	# for command line quote the single quote

  $CRYPTPW = `/usr/sbin/slappasswd -s '$TMPPW'` or die ("slappasswd failed!\n");
  chomp $CRYPTPW;

  $CRYPTPW =~ s/\r//g;
  $CRYPTPW =~ s/\n//g;

  $EMAIL = "\L$SHORTEMAIL\@$env::domain\E";

  $FULLNAME = "$FIRSTNAME $MIDDLENAME $LASTNAME";
  $FULLNAME =~ s/^ *//g; $FULLNAME =~ s/ *$//g; $FULLNAME =~ s/  / /g;

  my $INITIALS  = "";
  $INITIALS .= substr ($FIRSTNAME, 0, 1) if $FIRSTNAME;
  $INITIALS .= substr ($MIDDLENAME, 0, 1) if $MIDDLENAME;
  $INITIALS .= substr ($LASTNAME, 0, 1) if $LASTNAME;

  my $HEXINIT = "";
  foreach my $c ( unpack ("C*", $INITIALS) ) {
    $HEXINIT .= sprintf ("%02x,00,", $c);
  }
  $HEXINIT .= "00,00";

  my $HEXNAME = "";
  foreach my $c ( unpack ("C*", $FULLNAME) ) {
    $HEXNAME .= sprintf ("%02x,00,", $c);
  }
  $HEXNAME .= "00,00";

  $FULLNAMEBAK = $FULLNAME;

  $FULLNAME = pack ("U*", unpack ("C*", $FULLNAME));
  $FIRSTNAME = pack ("U*", unpack ("C*", $FIRSTNAME)) if $FIRSTNAME;
  $MIDDLENAME = pack ("U*", unpack ("C*", $MIDDLENAME)) if $MIDDLENAME;
  $LASTNAME = pack ("U*", unpack ("C*", $LASTNAME));

  $FULLNAME64 = `echo -n "$FULLNAME" | base64 -e`;
  $FIRSTNAME64 = `echo -n "$FIRSTNAME" | base64 -e` if $FIRSTNAME;
  $MIDDLENAME64 = `echo -n "$MIDDLENAME" | base64 -e` if $MIDDLENAME;
  $LASTNAME64 = `echo -n "$LASTNAME" | base64 -e`;

  $FULLNAME64 =~ /^([a-zA-Z0-9+\/=]*)/; $FULLNAME64 = $1;
  if ($FIRSTNAME) {
    $FIRSTNAME64 =~ /^([a-zA-Z0-9+\/=]*)/; $FIRSTNAME64 = $1;
  }
  if ($MIDDLENAME) {
    $MIDDLENAME64 =~ /^([a-zA-Z0-9+\/=]*)/; $MIDDLENAME64 = $1;
  }
  $LASTNAME64 =~ /^([a-zA-Z0-9+\/=]*)/; $LASTNAME64 = $1;

  if (-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME") {
    $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME`;
    if (-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME") {
      die ("Cannot delete existing file $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME!\n");
    }
  }

  $cmd  = qq|/bin/cat > $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME << EOF\n|
        . qq|dn: uid=$LOGINNAME, ou=users, $env::ldapbase\n|
        . qq|cn:: $FULLNAME64\n|;
  $cmd .= qq|givenName:: $FIRSTNAME64\n| if $FIRSTNAME;
  $cmd .= qq|initials:: $MIDDLENAME64\n| if $MIDDLENAME;
  $cmd .= qq|sn:: $LASTNAME64\n|
        . qq|uid: $LOGINNAME\n|
        . qq|uidNumber: $TMPUID\n|
        . qq|gidNumber: $TMPGID\n|
        . qq|homeDirectory: $HOMEDIR\n|
        . qq|loginShell: /bin/bash\n|
        . qq|userPassword: $CRYPTPW\n|
        . qq|mail: $EMAIL\n|
        . qq|objectClass: person\n|
        . qq|objectClass: inetOrgPerson\n|
        . qq|objectClass: posixAccount\n|
        . qq|EOF\n|
        . qq|chmod 600 $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME\n|;

  $TMPOUT = `$cmd`;

  if (!(-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME")) {
    die ("Cannot create file $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME!\n");
  }

  $TMPOUT = `/usr/bin/ldapadd -ZZ -a -c -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver`;

  print "created user entry in LDAP\n";

  $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME`;

  if (-e $HOMEDIR) {
    if (-d $HOMEDIR) {
      $TMPOUT = `chmod 700 "$HOMEDIR"`;
    }
    else {
      die "An Object $HOMEDIR does exist, but isn't a directory!\n";
    }
  }
  else {
    my $HOMEDIR_PATH = $HOMEDIR;
    $HOMEDIR_PATH =~ s|/[^/]*$||;	# strip last directory from path
    if (!(-e $HOMEDIR_PATH)) {
      print "Warning: Path $HOMEDIR_PATH to home directory doesn't exist\n"
            . "non-existing parents will be created automatically\n";
    }
    $TMPOUT = `/bin/mkdir -p -m 700 "$HOMEDIR"`;
  }

  $TMPOUT = `/bin/mkdir -m 700 "$HOMEDIR/mail"`;
  $TMPOUT = `/bin/mkdir -m 700 "$HOMEDIR/$env::mydocuments"`;

  print "created home directory and subdirectories\n";

  $REGTEMPLATE = `/bin/cat $env::DATAPATH/regtemplate.reg`;

  $REGTEMPLATE =~ s/#LOGINNAME#/$LOGINNAME/gs;
  $REGTEMPLATE =~ s/#FULLNAME#/$FULLNAMEBAK/gs;
  $REGTEMPLATE =~ s/#EMAIL#/$EMAIL/gs;
  $REGTEMPLATE =~ s/#DOMAIN#/$env::domain/gs;
  $REGTEMPLATE =~ s/#HEXINITIALS#/$HEXINIT/gs;
  $REGTEMPLATE =~ s/#HEXFULLNAME#/$HEXNAME/gs;
  $REGTEMPLATE =~ s/#FQHOSTNAME#/$env::fqhostname/gs;
  $REGTEMPLATE =~ s/#HOSTNAME#/$env::hostname/gs;
  $REGTEMPLATE =~ s/#IMAPSERVER#/$env::imapserver/gs;
  $REGTEMPLATE =~ s/#SMTPSERVER#/$env::smtpserver/gs;
  $REGTEMPLATE =~ s/#LDAPSERVER#/$env::ldapserver/gs;
  $REGTEMPLATE =~ s/#MAILSERVERNAME#/$env::mailservername/gs;
  $REGTEMPLATE =~ s/#ADDRESSBOOK#/$env::addressbook/gs;
  $REGTEMPLATE =~ s/#PROXYSERVER#/$env::proxyserver/gs;
  $REGTEMPLATE =~ s/#PROXYPORT#/$env::proxyport/gs;
  $REGTEMPLATE =~ s/#TEMPLATEDRIVE#/$env::templatedrive/gs;
  $REGTEMPLATE =~ s/#HOMEDRIVE#/$env::homedrive/gs;
  $REGTEMPLATE =~ s/#MYDOCUMENTS#/$env::mydocuments/gs;
  $REGTEMPLATE =~ s/#LDAPBASE#/$env::ldapbase/gs;

  open (REGFILE, "> $HOMEDIR/registry.reg");
  print REGFILE $REGTEMPLATE;
  close REGFILE;

  print "created registry update script\n";

  $LIST = &getpwnam ($LOGINNAME);
  ($UNAME, $CRYPTPW, $UID, $GID, $a, $b, $c, $d, $SHELL, $e, $TMPDIR) = @$LIST;
  while (!$UNAME) {
    $TMPOUT = `sleep 1`;
    print "Waiting for NSS cache to retrieve user id\n";
    $LIST = &getpwnam ($LOGINNAME);
    ($UNAME, $CRYPTPW, $UID, $GID, $a, $b, $c, $d, $SHELL, $e, $TMPDIR) = @$LIST;
  }

  $TMPOUT = `/bin/chown -R $LOGINNAME.users $HOMEDIR`;

  if (!(-d $HOMEDIR)) {
    die ("Home Directory creation failed!\n");
  }

  my $cmd = "cat << EOF | /usr/bin/smbpasswd -s -a $LOGINNAME\n$SMBPW\n$SMBPW\nEOF";
  $TMPOUT = `$cmd` or die ("creation of samba account failed!\n");

  print "Windows account created\n";

  &createmailbox ($LOGINNAME, $TMPPW);

  if (-e "$env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME") {
    $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME`;
    if (-e "$env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME") {
      die ("Cannot delete existing file $env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME!\n");
    }
  }

  $cmd  = qq|/bin/cat > $env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME << EOF\n|
        . qq|dn: sendmailMTAKey=$SHORTEMAIL, ou=aliases, $env::ldapbase\n|
        . qq|sendmailMTAAliasValue: $LOGINNAME\n|
        . qq|sendmailMTAKey: $SHORTEMAIL\n|
        . qq|sendmailMTAAliasGrouping: aliases\n|
        . qq|sendmailMTACluster: Mailservers\n|
        . qq|sendmailMTAHost: $env::fqhostname\n|
        . qq|objectClass: sendmailMTAAliasObject\n|
        . qq|EOF\n|
        . qq|chmod 600 $env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME\n|;

  $TMPOUT = `$cmd`;

  if (!(-e "$env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME")) {
    die ("Cannot create file $env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME!\n");
  }

  $TMPOUT = `/usr/bin/ldapadd -ZZ -a -c -f $env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver`;

  $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEALIASES-$LOGINNAME`;

  print "email account activated\n";

}

sub delete {

  my ($LOGINNAME) = @_;

  $LIST = &getpwnam ($LOGINNAME);
  ($UNAME, $CRYPTPW, $UID, $GID, $a, $b, $c, $d, $SHELL, $e, $HOMEDIR) = @$LIST;
  # suppress "possible typo: variable used only once" warning
  #$UNAME; $CRYPTPW; $UID; $GID; $a; $b; $c; $d; $SHELL; $e;

  if (!$HOMEDIR) {
    die ("User $LOGINNAME doesn't exist!\n");
  }

  if ((-e "$HOMEDIR.tgz") || (-e "$HOMEDIR-mail.tgz") || (-e "$HOMEDIR.ldif")) {
    die ("You have to remove $HOMEDIR.tgz, $HOMEDIR-mail.tgz and $HOMEDIR.ldif\n"
         . "so that I can create a backup.\n");
  }

  $TMPOUT = `/usr/bin/ldapsearch -x -b "$env::ldapbase" -LLL "(uid=$LOGINNAME)" > "$HOMEDIR.ldif"`;
  if (!(-e "$HOMEDIR.ldif")) {
    die ("Could not create $HOMEDIR.ldif!\n");
  }

  print "User LDAP entry saved\n";

  $DN = `grep -i '^dn:' "$HOMEDIR.ldif"`;
  $DN =~ /dn: (.*)/i;
  $DN = $1;

  $SHORTEMAIL = `grep -i '^mail:' "$HOMEDIR.ldif"`;
  $SHORTEMAIL =~ /mail: (.*)@.*/i;
  $SHORTEMAIL = $1;

  $TMPOUT = `tar czf $HOMEDIR.tgz $HOMEDIR 2> /dev/null`;
  if (!(-e "$HOMEDIR.tgz")) {
    die ("Could not create $HOMEDIR.tgz!\n");
  }

  print "home directory saved\n";

  if (!$env::maildir) {
    print "Variable maildir not defined - mailbox cannot be archived\n";
    print "Possibly mail is on a remote machine?\n";
  }
  else {
    $TMPOUT = `tar czf $HOMEDIR-mail.tgz $env::maildir/$LOGINNAME 2> /dev/null`;
    if (!(-e "$HOMEDIR-mail.tgz")) {
      die ("Could not create $HOMEDIR-mail.tgz!\n");
    }
    print "mailbox saved\n";
  }

  $cmd = "tar czf $HOMEDIR-squirrel.tgz";
  if (-e "$env::squirreldata/$LOGINNAME.pref") {
    $cmd .= " $env::squirreldata/$LOGINNAME.pref";
  }
  if (-e "$env::squirreldata/$LOGINNAME.abook") {
    $cmd .= " $env::squirreldata/$LOGINNAME.abook";
  }
  if ($cmd =~ /\.tgz .+/) {
    $TMPOUT = `$cmd 2> /dev/null`;
    if (!(-e "$HOMEDIR-squirrel.tgz")) {
      die ("Could not create $HOMEDIR-mail.tgz!\n");
    }
  }

  print "squirrelmail preferences saved\n";

  &deletemailbox ($LOGINNAME);

  $TMPOUT = `/usr/bin/ldapdelete -ZZ -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver "sendmailMTAKey=$SHORTEMAIL, ou=aliases, $env::ldapbase"`;

  print "email account deactivated\n";

  $TMPOUT = `chmod 600 $HOMEDIR.tgz $HOMEDIR-mail.tgz $HOMEDIR-mail.tgz $HOMEDIR.ldif`;

  $TMPOUT = `/usr/bin/ldapdelete -ZZ -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver "$DN"`;

  print "User LDAP entry deleted\n";

  $TMPOUT = `/bin/rm -rf $HOMEDIR`;
  if (-e $HOMEDIR) {
    print "Could not delete $HOMEDIR. You have to do it manually.\n";
  }

  print "home directory deleted\n";

  $TMPOUT = `/bin/rm -f $env::squirreldata/$LOGINNAME.abook`;
  $TMPOUT = `/bin/rm -f $env::squirreldata/$LOGINNAME.pref`;
  if (   -e "$env::squirreldata/$LOGINNAME.abook"
      || -e "$env::squirreldata/$LOGINNAME.pref") {
    print "Could not delete SquirrelMail user files. You have to do it manually.\n";
  }

  print "squirrelmail preferences deleted\n";

}

sub unixpasswd {

  my ($LOGINNAME, $TMPPW, $DIRECTORY) = @_;

  if (!(($LOGINNAME eq "root") || ($LOGINNAME =~ /\$$/))) {

    $TMPPW =~ s/'/'\\''/g;
    $TMPPW =~ s/\r//g;
    $TMPPW =~ s/\n//g;

    $CRYPTPW = `/usr/sbin/slappasswd -s '$TMPPW'` or die ("slappasswd failed!\n");
    chomp $CRYPTPW;

    $CRYPTPW =~ s/\r//g;
    $CRYPTPW =~ s/\n//g;

    if (-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME") {
      $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME`;
      if (-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME") {
        die ("Cannot delete existing file $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME!\n");
      }
    }

    $cmd = qq|/bin/cat > $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME << EOF\n|
         . qq|dn: uid=$LOGINNAME, ou=$DIRECTORY, $env::ldapbase\n|
         . qq|userPassword: $CRYPTPW\n|
         . qq|EOF\n|
         . qq|chmod 600 $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME\n|;

    $TMPOUT = `$cmd`;

    if (!(-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME")) {
      die ("Cannot create file $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME!\n");
    }

    $TMPOUT = `/usr/bin/ldapmodify -ZZ -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver` or die ("ldapmodify failed!\n");

    $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME`;

  } # else: $LOGINNAME = root, root cannot change his unixpassword via samba
    #       or $LOGINNAME = ...$, that's a windows machine which doesn't have
    #       a unix password / login posibility
}

sub unsafepasswd {

  my ($LOGINNAME, $TMPPW, $DIRECTORY) = @_;

  if (!(($LOGINNAME eq "root") || ($LOGINNAME =~ /\$$/))) {

    $TMPPW =~ s/'/'\\''/g;
    $TMPPW =~ s/\r//g;
    $TMPPW =~ s/\n//g;

    $CRYPTPW = `/usr/sbin/slappasswd -s '$TMPPW'` or die ("slappasswd failed!\n");
    chomp $CRYPTPW;

    $CRYPTPW =~ s/\r//g;
    $CRYPTPW =~ s/\n//g;

    if (-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME") {
      $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME`;
      if (-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME") {
        die ("Cannot delete existing file $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME!\n");
      }
    }

    $cmd = qq|/bin/cat > $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME << EOF\n|
         . qq|dn: uid=$LOGINNAME, ou=$DIRECTORY, $env::ldapbase\n|
         . qq|userPassword: $CRYPTPW\n|
         . qq|EOF\n|
         . qq|chmod 600 $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME\n|;

    $TMPOUT = `$cmd`;

    if (!(-e "$env::DATAPATH/tmp/CREATEUSER-$LOGINNAME")) {
      die ("Cannot create file $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME!\n");
    }

    $TMPOUT = `/usr/bin/ldapmodify -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn"` or die ("ldapmodify failed!\n");

    $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/CREATEUSER-$LOGINNAME`;

  } # else: $LOGINNAME = root, root cannot change his unixpassword via samba
    #       or $LOGINNAME = ...$, that's a windows machine which doesn't have
    #       a unix password / login posibility
}

# procedures for address book management

sub getattribute {

  my ($ATTR, $TMPOUT) = @_;

  if ($TMPOUT =~ /\n$ATTR\: ([^\n]*)/si) {
    $VALUE = $1;
  }
  elsif ($TMPOUT =~ /\n$ATTR\:\: ([^\n]*)/si) {
    my $TMP = $1;
    $VALUE = `echo $TMP | /usr/local/bin/base64 -d`;
    $VALUE = pack ("C*", unpack ("U*", $VALUE));
  }
  else {
    $VALUE = undef;
  }

  return $VALUE;

}

sub setattribute {

  my ($ATTR, $TMPOUT) = @_;

  $STR = "$ATTR:";

  if ($TMPOUT =~ /[^a-zA-Z0-9 \/!?'"<>=()._+-]/) {
    # encode in utf-8 / base64
    $TMPOUT = pack ("U*", unpack ("C*", $TMPOUT));
    $TMPOUT64 = `echo -n "$TMPOUT" | base64 -e`;
    $TMPOUT64 =~ /^([a-zA-Z0-9+\/=]*)/; $TMPOUT64 = $1;
    $STR .= ": $TMPOUT64";
  }
  else {
    # return unencoded
    $STR .= " $TMPOUT";
  }

  return $STR;

}

sub setentry {

  my ($VALUELIST, $NEWVALUELIST, $ATTRLIST) = @_;

  if (-e "$env::DATAPATH/tmp/MODIFYENTRY") {
    $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/MODIFYENTRY`;
    if (-e "$env::DATAPATH/tmp/MODIFYENTRY") {
      die ("Cannot delete existing file $env::DATAPATH/tmp/MODIFYENTRY!\n");
    }
  }

  $cmd = qq|/bin/cat > $env::DATAPATH/tmp/MODIFYENTRY << EOF\n|
       . "dn: " . $$VALUELIST{"dn"} . "\n";

  foreach $ATTR (@$ATTRLIST) {

    $TMPVAL = $$NEWVALUELIST{$ATTR};

    if ($TMPVAL) {
      if ($TMPVAL eq ".") {
        if ($$VALUELIST{$ATTR}) {
          $cmd .= "delete: $ATTR\n-\n";
        }
      }
      else {
        if ($$VALUELIST{$ATTR}) {
          $cmd .= "replace: $ATTR\n" . &setattribute ($ATTR, $TMPVAL) . "\n-\n";
        }
        else {
          $cmd .= "add: $ATTR\n" . &setattribute ($ATTR, $TMPVAL) . "\n-\n";
        }
      }
    }

  }

  $cmd .= qq|EOF\n|
        . qq|chmod 600 $env::DATAPATH/tmp/MODIFYENTRY\n|;

  $TMPOUT = `$cmd`;

  if (!(-e "$env::DATAPATH/tmp/MODIFYENTRY")) {
    die ("Cannot create file $env::DATAPATH/tmp/MODIFYENTRY!\n");
  }

  $TMPOUT = `/usr/bin/ldapmodify -ZZ -f $env::DATAPATH/tmp/MODIFYENTRY -x -y $env::DATAPATH/slappasswd -D "$env::ldaprootdn" -h $env::ldapserver` or die ("ldapmodify failed!\n");

  $TMPOUT = `/bin/rm -f $env::DATAPATH/tmp/MODIFYENTRY`;

}

sub getentry {

  my ($SEARCHMASK, $ATTRLIST) = @_;

  $TMPOUT = `/usr/bin/ldapsearch -x -b "$env::ldapbase" -LLL "$SEARCHMASK"`;

  $TMPOUT = "\n" . $TMPOUT;

  foreach $ATTR (@$ATTRLIST) {
    $VALUELIST{$ATTR} = &getattribute ($ATTR, $TMPOUT);
  }

  return %VALUELIST;

}

sub getpersonattributes {
  my @ATTRLIST = ("dn", "title", "o", "ou", "l", "st", "c", "postalAddress", "postalCode", "physicalDeliveryOfficeName", "telephoneNumber", "mobile", "facsimileTelephoneNumber", "homePostalAddress", "homePhone", "otherFacsimileTelephoneNumber", "URL", "comment");
  return @ATTRLIST;
}

sub getmachineattributes {
  my @ATTRLIST = ("dn", "description");
  return @ATTRLIST;
}

sub ssh_trustaccount {
  my ($remoteuser, $remote, $localuser) = @_;

  my $LIST = &getpwnam ($localuser);
  my ($UNAME, $CRYPTPW, $UID, $GID, $a, $b, $c, $d, $SHELL, $e, $home) = @$LIST;
  # RSA key of THIS machine
  my $hostname = `hostname -f`;
  $hostname =~ /([a-zA-Z0-9._-]*)/; $hostname = $1;
  my $rsalocal = `cat /etc/ssh/ssh_host_rsa_key.pub | sed 's/\\([^=]*==*\\) .*/$hostname \\1/'`;
  $rsalocal =~ /(.*)/; $rsalocal = $1;

  # generate key pair for local user
  # (who will be the trusted user on the REMOTE machine)
  if (! -d "$home/.ssh") {
    mkdir ("$home/.ssh", 0700);
  }
  if ((! -e "$home/.ssh/id_rsa") || (! -e "$home/.ssh/id_rsa.pub")) {
    system ("ssh-keygen -b 2048 -t rsa -N \"\" -f $home/.ssh/id_rsa");
    system ("chown -R $localuser $home/.ssh");
  }
  my $rsauser = `cat $home/.ssh/id_rsa.pub`;
  $rsauser =~ /(.*)/; $rsauser = $1;
  $rsauser =~ /.* ([^ ]*)$/;
  my $rsaid = $1;

  # open connection; prompt for password and host authenticity, if trust
  # is not already established
  $pid = open2(*R, *W, "su $localuser -c \"ssh -l $remoteuser $remote\"");

  # add host and user public RSA key to the appropriate files
  print W qq|if [ ! -d "\$HOME/.ssh" ]; |
	  . qq|then mkdir -m 700 "\$HOME/.ssh"; |
	  . qq|fi\n|;
  print W qq|if grep $hostname \$HOME/.ssh/known_hosts > /dev/null 2>&1; |
	  . qq|then echo "Host already known"; else cat << "EOF" |
	  . qq|>> \$HOME/.ssh/known_hosts\n$rsalocal\nEOF\n|
	  . qq|fi\n|;
  print W qq|if grep $rsaid \$HOME/.ssh/authorized_keys > /dev/null 2>&1; |
	  . qq|then echo "User already known"; else cat << "EOF" |
	  . qq|>> \$HOME/.ssh/authorized_keys\n$rsauser\nEOF\n|
	  . qq|fi\n|;
  print W qq|chmod 600 \$HOME/.ssh/*\n|;
  print W "exit\n";
  while ($got = <R>) {
    print ":: $got";
  }
}

return 1;
